@@ -41,7 +41,7 @@
 #' @examples
 #' data("BSG2014")
 #'
-#' # run fast and robust bootstrap test
+#' # run fast-and-robust bootstrap test
 #' robust_boot <- test_mediation(BSG2014,
 #'                               x = "ValueDiversity",
 #'                               y = "TeamCommitment",
@@ -120,13 +120,14 @@ density_plot.setup_density_plot <- function(object, ...) {
   # define aesthetic mappings for density estimate, point estimate and
   # confidence interval
   if(object$have_methods) {
-    mapping_density <- aes_string(x = "ab", y = "Density", color = "Method")
+    mapping_density <- aes_string(x = "Indirect", y = "Density",
+                                  color = "Method")
     mapping_estimate <- aes_string(xintercept = "Estimate", color = "Method")
     mapping_ci <- aes_string(xmin = "Lower", xmax = "Upper",
                              ymin = -Inf, ymax = Inf,
                              fill = "Method")
   } else {
-    mapping_density <- aes_string(x = "ab", y = "Density")
+    mapping_density <- aes_string(x = "Indirect", y = "Density")
     mapping_estimate <- aes_string(xintercept = "Estimate")
     mapping_ci <- aes_string(xmin = "Lower", xmax = "Upper",
                              ymin = -Inf, ymax = Inf)
