@@ -30,12 +30,12 @@
 #' @return An object of class \code{"setup_density_plot"} with the following
 #' components:
 #' \item{density}{a data frame containing the values of the indirect effect
-#' where the density is estimated (column \code{ab}), and the estimated density
-#' values (column \code{Density}).  In case of a model with multiple indirect
-#' effects, there is a column \code{Effect} that indicates the different
-#' indirect effects.  If a list of \code{"\link{test_mediation}"} objects has
-#' been supplied, there is also a column \code{Method}, which takes the names
-#' or indices of the list elements to indicate the different methods.}
+#' where the density is estimated (column \code{Indirect}), and the estimated
+#' density values (column \code{Density}).  In case of a model with multiple
+#' indirect effects, there is a column \code{Effect} that indicates the
+#' different indirect effects.  If a list of \code{"\link{test_mediation}"}
+#' objects has been supplied, there is also a column \code{Method}, which takes
+#' the names or indices of the list elements to indicate the different methods.}
 #' \item{ci}{a data frame consisting of column \code{Estimate} containing the
 #' point estimates, column \code{Lower} for the lower confidence limit, and
 #' column \code{Upper} for the upper confidence limit.  In case of a model with
@@ -65,19 +65,18 @@
 #' @examples
 #' data("BSG2014")
 #'
-#' # run fast and robust bootstrap test
-#' robust_boot <- test_mediation(BSG2014,
-#'                               x = "ValueDiversity",
-#'                               y = "TeamCommitment",
-#'                               m = "TaskConflict",
-#'                               robust = TRUE)
+#' # run fast-and-robust bootstrap test
+#' test <- test_mediation(BSG2014,
+#'                        x = "ValueDiversity",
+#'                        y = "TeamCommitment",
+#'                        m = "TaskConflict")
 #'
 #' # set up information for plot
-#' setup <- setup_density_plot(robust_boot)
+#' setup <- setup_density_plot(test)
 #'
 #' # plot only density and confidence interval
 #' ggplot() +
-#'   geom_density(aes(x = ab, y = Density), data = setup$density,
+#'   geom_density(aes(x = Indirect, y = Density), data = setup$density,
 #'                stat = "identity") +
 #'   geom_rect(aes(xmin = Lower, xmax = Upper,
 #'                 ymin = -Inf, ymax = Inf),
@@ -98,46 +97,39 @@ setup_density_plot <- function(object, ...) UseMethod("setup_density_plot")
 
 setup_density_plot.boot_test_mediation <- function(object, ...) {
   # initialization
-  nr_indirect <- length(object$fit$x) * length(object$fit$m)
-  have_effects <- nr_indirect > 1L
-  contrast <- object$fit$contrast          # only implemented for regression fit
+  fit <- object$fit
+  have_effects <- !is.null(fit$model) && fit$model != "simple"
+  contrast <- fit$contrast                 # only implemented for regression fit
   have_contrast <- is.character(contrast)  # but this always works
   # extract point estimate and confidence interval
-  ab <- object$ab
+  indirect <- object$indirect
   ci <- object$ci
+  # extract bootstrap replicates of the indirect effects
+  if (inherits(fit, "reg_fit_mediation")) {
+    boot_indirect <- extract_boot(fit, boot = object$reps)$indirect
+  } else if (inherits(fit, "cov_fit_mediation")) {
+    boot_indirect <- object$reps$t[, 5L, drop = FALSE]
+  } else stop("not implemented for this type of model fit")
   # extract information to be plotted
   if (have_effects) {
     # information on indirect effects
-    effect_labels <- names(ab)
+    effect_labels <- names(indirect)
     effects <- factor(effect_labels, levels = effect_labels)
-    # compute bootstrap densities
-    indices <- seq_len(1L + nr_indirect)
-    pdf_list <- lapply(indices, function(j, x) density(x[, j], na.rm = TRUE),
-                       x = object$reps$t)
-    # add densities of contrasts (if applicable)
-    if (have_contrast) {
-      bootstrap_contrasts <- get_contrasts(object$reps$t[, indices[-1L]],
-                                           type = contrast)
-      n_contrasts <- ncol(bootstrap_contrasts)
-      bootstrap_pdf_list <- lapply(seq_len(n_contrasts),
-                                   function(j, x) density(x[, j], na.rm = TRUE),
-                                   x = bootstrap_contrasts)
-      pdf_list <- c(pdf_list, bootstrap_pdf_list)
-    }
-    # construct data frame containing bootstrap densities
-    density_list <- mapply(function(pdf, effect) {
-      data.frame(Effect = effect, ab = pdf$x, Density = pdf$y)
-    }, pdf = pdf_list, effect = effects, SIMPLIFY = FALSE, USE.NAMES = FALSE)
+    # compute bootstrap densities and construct data frame
+    density_list <- lapply(effects, function(effect) {
+      pdf <- density(boot_indirect[, effect], na.rm = TRUE)
+      data.frame(Effect = effect, Indirect = pdf$x, Density = pdf$y)
+    })
     density <- do.call(rbind, density_list)
     # construct data frame containing confidence interval
-    ci <- data.frame(Effect = effects, Estimate = unname(ab),
+    ci <- data.frame(Effect = effects, Estimate = unname(indirect),
                      Lower = unname(ci[, 1L]), Upper = unname(ci[, 2L]))
   } else {
     # construct data frame containing bootstrap density
-    pdf <- density(object$reps$t[, 1L], na.rm = TRUE)
-    density <- data.frame(ab = pdf$x, Density = pdf$y)
+    pdf <- density(boot_indirect, na.rm = TRUE)
+    density <- data.frame(Indirect = pdf$x, Density = pdf$y)
     # construct data frame containing confidence interval
-    ci <- data.frame(Estimate = ab, Lower = ci[1L], Upper = ci[2L])
+    ci <- data.frame(Estimate = indirect, Lower = ci[1L], Upper = ci[2L])
   }
   # return density and confidence interval
   out <- list(density = density, ci = ci, test = "boot", level = object$level,
@@ -154,21 +146,21 @@ setup_density_plot.boot_test_mediation <- function(object, ...) {
 setup_density_plot.sobel_test_mediation <- function(object, grid = NULL,
                                                     level = 0.95, ...) {
   # initializations
-  level <- rep(as.numeric(level), length.out = 1)
+  level <- rep(as.numeric(level), length.out = 1L)
   if (is.na(level) || level < 0 || level > 1) level <- formals()$level
   # extract point estimate and standard error
-  ab <- object$fit$ab
+  indirect <- object$fit$indirect
   se <- object$se
   # construct data frame containing x- and y-values for the density
   if (is.null(grid)) {
     n_grid <- formals(density.default)$n  # same number of points as density()
-    grid <- seq(ab - 3 * se, ab + 3 * se, length.out = n_grid)
+    grid <- seq(indirect - 3 * se, indirect + 3 * se, length.out = n_grid)
   } else grid <- as.numeric(grid)
-  y <- dnorm(grid, mean = ab, sd = se)
-  density <- data.frame(ab = grid, Density = y)
+  y <- dnorm(grid, mean = indirect, sd = se)
+  density <- data.frame(Indirect = grid, Density = y)
   # construct data frame containing confidence interval
-  ci <- confint_z(ab, se, level = level, alternative = object$alternative)
-  ci <- data.frame(Estimate = ab, Lower = ci[1], Upper = ci[2])
+  ci <- confint_z(indirect, se, level = level, alternative = object$alternative)
+  ci <- data.frame(Estimate = indirect, Lower = ci[1L], Upper = ci[2L])
   # return density and confidence interval
   out <- list(density = density, ci = ci, test = "sobel", level = level,
               have_effects = FALSE, have_methods = FALSE)
